package de.jottyfan.camporganizer.module.admin;

import static de.jottyfan.camporganizer.db.jooq.Tables.T_DOCUMENT;

import java.io.IOException;
import java.util.List;

import javax.validation.Valid;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.jooq.exception.DataAccessException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.jottyfan.camporganizer.db.jooq.enums.EnumDocument;
import de.jottyfan.camporganizer.module.mail.MailBean;
import de.jottyfan.camporganizer.module.mail.MailRepository;

/**
 *
 * @author jotty
 *
 */
@Service
public class AdminService {
	private static final Logger LOGGER = LogManager.getLogger(AdminService.class);

	@Autowired
	private MailRepository mailRepository;

	@Autowired
	private AdminRepository adminRepository;

	public void sendMail(@Valid MailBean bean) {
		mailRepository.sendMail(bean);
	}

	/**
	 * get all documents
	 *
	 * @return a list of documents; an empty one at least
	 */
	public List<DocumentBean> getAllDocuments() {
		return adminRepository.getAllDocuments();
	}

	/**
	 * get document from the database
	 *
	 * @param id the ID of the document
	 * @return the document or null
	 */
	public DocumentBean getDocument(Integer id) {
		return adminRepository.getDocument(id);
	}

	/**
	 * upsert the document bean
	 *
	 * @param bean the bean
	 * @return the number of affected database lines
	 */
	public Integer updateDocument(@Valid DocumentBean bean) {
		try {
			bean.encodeUpload();
		} catch (IOException e) {
			LOGGER.error(e.getMessage());
		}
		return adminRepository.upsert(bean);
	}

	/**
	 * delete document with id from the database
	 *
	 * @param id the ID of the document
	 * @return the number of affected database lines
	 */
	public Integer deleteDocument(Integer id) {
		return adminRepository.deleteDocument(id);
	}

	/**
	 * get all locations
	 *
	 * @return the locations
	 */
	public List<LocationBean> getLocations() {
		return adminRepository.getLocations();
	}

	/**
	 * get the location of id
	 *
	 * @param id the id
	 * @return the location or null
	 */
	public LocationBean getLocation(Integer id) {
		return adminRepository.getLocation(id);
	}

	/**
	 * upsert the bean
	 *
	 * @param bean the bean
	 */
	public void upsertLocation(@Valid LocationBean bean) {
		adminRepository.upsertLocation(bean);
	}

	/**
	 * delete the location
	 *
	 * @param id the ID of the location
	 */
	public void deleteLocation(Integer id) {
		// TODO: if a location is still in use by a camp, forbid deleting it
	     adminRepository.deleteLocation(id);
	}

	/**
	 * get all documents that fit to the location definitions
	 *
	 * @return the location documents
	 */
	public List<DocumentBean> getLocationDocuments() {
		return adminRepository.getAllDocumentsWith(T_DOCUMENT.DOCTYPE.eq(EnumDocument.location));
	}
}
